unit InfoDialog;

{$IMPORTEDDATA OFF}

interface

uses NLib, NForm, NGraphics, NDesktop, Player;

type
  TInfoDialog = class(TDialog)
    Icon: TIcon;
    Player: TPlayer;
    MTime: Int64;
    Size: Int64;
    constructor Create; override;
    destructor Destroy; override;
    procedure Init(var FocusId: Integer); override;
    procedure UpdateInfo;
  end;

implementation

uses NSys;

{$I dialogid.inc}

function BytesFormat(Value: Int64): string;
var
  s: string;
begin
  Result := UnitFormat(Value);
  s := Copy(Result, 1, Length(Result)-1);
  case Result[Length(Result)] of
    'G': Result := s + ' GB';
    'M': Result := s + ' MB';
    'K': Result := s + ' KB';
  else
    Result := Result + ' bytes';
  end;
end;

// to calcurate at runtime
function CalcRatio(A, B: Double): Double;
begin
  Result := A / B;
end;

function RatioFormat(X, Y: Integer): string;
var
  r: Double;
  g: Cardinal;
begin
  if Y = 0 then begin
    Result := '';
    Exit;
  end;
  r := X / Y;
  if r = CalcRatio(16, 9) then
    Result := '16:9'
  else if r = CalcRatio(4, 3) then
    Result := '4:3'
  else begin
    g := GCD(X, Y);
    Result := IStr(Cardinal(X) div g) + ':' + IStr(Cardinal(Y) div g);
  end;
end;

constructor TInfoDialog.Create;
begin
  inherited;
  TemplateName := 'INFODIALOG';
end;

destructor TInfoDialog.Destroy;
begin
  if Assigned(Icon) then
    Icon.Free;
  inherited;
end;

procedure TInfoDialog.Init(var FocusId: Integer);
begin
  inherited;
  UpdateInfo;
end;

procedure TInfoDialog.UpdateInfo;
var
  t: TTime;
  hasvideo: Boolean;
begin
  if not Assigned(Icon) then
    Icon := TIcon.Create;
  if Player.SourceType = pstUrl then
    LoadFileIcon(Icon, 'shell32.dll', 13)
  else
    LoadFileIcon(Icon, Player.Path);
  SetIcon(IDC_ICON, Icon);

  Items[IDC_FILENAME].SetText(BaseName(Player.Path));

  if Player.SourceType = pstUrl then
    Items[IDC_LOCATION].SetText(Player.Path)
  else
    Items[IDC_LOCATION].SetText(DirName(AbsPath(Player.Path)));

  if Player.Len > -1 then
    Items[IDC_LENGTH].SetText(
        Format('{}.{:03}', [FormatTime(Player.Len), Player.Len mod 1000]))
  else
    Items[IDC_LENGTH].SetText('');

  Items[IDC_FORMAT].SetText(Player.MediaInfo.Container);

  hasvideo := Player.HasVideo or (Player.MediaInfo.VideoWidth > 0);
  Items[IDC_SIZELABEL].Enable(hasvideo);
  Items[IDC_FPSLABEL].Enable(hasvideo);
  Items[IDC_ASPECTRATIOLABEL].Enable(hasvideo);
  Items[IDC_VIDEOBITRATELABEL].Enable(hasvideo);
  Items[IDC_VIDEOCODECLABEL].Enable(hasvideo);
  with Player.MediaInfo do begin
    if hasvideo then
    begin
      Items[IDC_SIZE].SetText(Format('{}x{}', [VideoWidth, VideoHeight]));
      if VideoFrameRate = 0 then
        Items[IDC_FPS].SetText('')
      else
        Items[IDC_FPS].SetText(Format('{} fps', [FStr(VideoFrameRate, 3)]));
      Items[IDC_ASPECTRATIO].SetText(
        RatioFormat(Player.VideoAspect.cx, Player.VideoAspect.cy));
      if VideoBitRate = 0 then
        Items[IDC_VIDEOBITRATE].SetText('')
      else
        Items[IDC_VIDEOBITRATE].SetText(Format('{} kbps', [VideoBitRate div 1000]));
      Items[IDC_VIDEOCODEC].SetText(VideoCodec);
    end else
    begin
      Items[IDC_SIZE].SetText('');
      Items[IDC_FPS].SetText('');
      Items[IDC_ASPECTRATIO].SetText('');
      Items[IDC_VIDEOBITRATE].SetText('');
      Items[IDC_VIDEOCODEC].SetText('');
    end;

    if AudioChannels = 0 then
      Items[IDC_AUDIOCHANNELS].SetText('')
    else
      Items[IDC_AUDIOCHANNELS].SetText(IStr(AudioChannels));
    if AudioSampleRate = 0 then
      Items[IDC_AUDIOSAMPLERATE].SetText('')
    else
      Items[IDC_AUDIOSAMPLERATE].SetText(Format('{} Hz', [AudioSampleRate]));
    if AudioBitRate = 0 then
      Items[IDC_AUDIOBITRATE].SetText('')
    else
      Items[IDC_AUDIOBITRATE].SetText(Format('{} kbps', [AudioBitRate div 1000]));
    Items[IDC_AUDIOCODEC].SetText(AudioCodec);
  end;

  if Size = 0 then
  begin
    Items[IDC_FILESIZE].SetText('');
    Items[IDC_MODIFIED].SetText('');
  end
  else begin
    Items[IDC_FILESIZE].SetText(
        Format('{} ({} bytes)', [BytesFormat(Size), NumFormat(Size)]));
    t := LocalTime(MTime);
    Items[IDC_MODIFIED].SetText(
        Format('{:04}-{:02}-{:02} {:02}:{:02}:{:02}',
          [t.Year, t.Mon, t.MDay, t.Hour, t.Min, t.Sec]));
  end;
end;

end.
